﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace KludgeBox
{
    public static class Base62
    {
        private const string chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

        public static string ToBase62(long number)
        {
            if (number == 0)
                return "0";

            string result = string.Empty;
            bool isNegative = number < 0;
            number = Math.Abs(number);

            while (number > 0)
            {
                result = chars[(int)(number % 62)] + result;
                number /= 62;
            }

            if (isNegative)
                result = "-" + result;

            return result;
        }

        public static long FromBase62(string base62)
        {
            if (string.IsNullOrEmpty(base62))
                throw new ArgumentException("Input string is empty.");

            bool isNegative = false;
            if (base62[0] == '-')
            {
                isNegative = true;
                base62 = base62.Substring(1);
            }

            long result = 0;
            int power = 0;

            for (int i = base62.Length - 1; i >= 0; i--)
            {
                char c = base62[i];
                int charValue = chars.IndexOf(c);

                if (charValue == -1)
                    throw new ArgumentException("Invalid character in input string.");

                result += charValue * (long)Math.Pow(62, power);
                power++;
            }

            return isNegative ? -result : result;
        }
    }
}
