﻿using KludgeBox.Structs;
using System;
using System.Collections.Generic;

namespace KludgeBox;

/// <summary>
/// This class contains color constants created from standardized color names.
/// The standardized color set is based on the X11 and .NET color names.
/// </summary>
public static class Colors
{
    // Color names and values are derived from core/math/color_names.inc
    internal static readonly Dictionary<string, KColor> namedColors = new()
    {
        { "ALICEBLUE", AliceBlue },
        { "ANTIQUEWHITE", AntiqueWhite },
        { "AQUA", Aqua },
        { "AQUAMARINE", Aquamarine },
        { "AZURE", Azure },
        { "BEIGE", Beige },
        { "BISQUE", Bisque },
        { "BLACK", Black },
        { "BLANCHEDALMOND", BlanchedAlmond },
        { "BLUE", Blue },
        { "BLUEVIOLET", BlueViolet },
        { "BROWN", Brown },
        { "BURLYWOOD", Burlywood },
        { "CADETBLUE", CadetBlue },
        { "CHARTREUSE", Chartreuse },
        { "CHOCOLATE", Chocolate },
        { "CORAL", Coral },
        { "CORNFLOWERBLUE", CornflowerBlue },
        { "CORNSILK", Cornsilk },
        { "CRIMSON", Crimson },
        { "CYAN", Cyan },
        { "DARKBLUE", DarkBlue },
        { "DARKCYAN", DarkCyan },
        { "DARKGOLDENROD", DarkGoldenrod },
        { "DARKGRAY", DarkGray },
        { "DARKGREEN", DarkGreen },
        { "DARKKHAKI", DarkKhaki },
        { "DARKMAGENTA", DarkMagenta },
        { "DARKOLIVEGREEN", DarkOliveGreen },
        { "DARKORANGE", DarkOrange },
        { "DARKORCHID", DarkOrchid },
        { "DARKRED", DarkRed },
        { "DARKSALMON", DarkSalmon },
        { "DARKSEAGREEN", DarkSeaGreen },
        { "DARKSLATEBLUE", DarkSlateBlue },
        { "DARKSLATEGRAY", DarkSlateGray },
        { "DARKTURQUOISE", DarkTurquoise },
        { "DARKVIOLET", DarkViolet },
        { "DEEPPINK", DeepPink },
        { "DEEPSKYBLUE", DeepSkyBlue },
        { "DIMGRAY", DimGray },
        { "DODGERBLUE", DodgerBlue },
        { "FIREBRICK", Firebrick },
        { "FLORALWHITE", FloralWhite },
        { "FORESTGREEN", ForestGreen },
        { "FUCHSIA", Fuchsia },
        { "GAINSBORO", Gainsboro },
        { "GHOSTWHITE", GhostWhite },
        { "GOLD", Gold },
        { "GOLDENROD", Goldenrod },
        { "GRAY", Gray },
        { "GREEN", Green },
        { "GREENYELLOW", GreenYellow },
        { "HONEYDEW", Honeydew },
        { "HOTPINK", HotPink },
        { "INDIANRED", IndianRed },
        { "INDIGO", Indigo },
        { "IVORY", Ivory },
        { "KHAKI", Khaki },
        { "LAVENDER", Lavender },
        { "LAVENDERBLUSH", LavenderBlush },
        { "LAWNGREEN", LawnGreen },
        { "LEMONCHIFFON", LemonChiffon },
        { "LIGHTBLUE", LightBlue },
        { "LIGHTCORAL", LightCoral },
        { "LIGHTCYAN", LightCyan },
        { "LIGHTGOLDENROD", LightGoldenrod },
        { "LIGHTGRAY", LightGray },
        { "LIGHTGREEN", LightGreen },
        { "LIGHTPINK", LightPink },
        { "LIGHTSALMON", LightSalmon },
        { "LIGHTSEAGREEN", LightSeaGreen },
        { "LIGHTSKYBLUE", LightSkyBlue },
        { "LIGHTSLATEGRAY", LightSlateGray },
        { "LIGHTSTEELBLUE", LightSteelBlue },
        { "LIGHTYELLOW", LightYellow },
        { "LIME", Lime },
        { "LIMEGREEN", LimeGreen },
        { "LINEN", Linen },
        { "MAGENTA", Magenta },
        { "MAROON", Maroon },
        { "MEDIUMAQUAMARINE", MediumAquamarine },
        { "MEDIUMBLUE", MediumBlue },
        { "MEDIUMORCHID", MediumOrchid },
        { "MEDIUMPURPLE", MediumPurple },
        { "MEDIUMSEAGREEN", MediumSeaGreen },
        { "MEDIUMSLATEBLUE", MediumSlateBlue },
        { "MEDIUMSPRINGGREEN", MediumSpringGreen },
        { "MEDIUMTURQUOISE", MediumTurquoise },
        { "MEDIUMVIOLETRED", MediumVioletRed },
        { "MIDNIGHTBLUE", MidnightBlue },
        { "MINTCREAM", MintCream },
        { "MISTYROSE", MistyRose },
        { "MOCCASIN", Moccasin },
        { "NAVAJOWHITE", NavajoWhite },
        { "NAVYBLUE", NavyBlue },
        { "OLDLACE", OldLace },
        { "OLIVE", Olive },
        { "OLIVEDRAB", OliveDrab },
        { "ORANGE", Orange },
        { "ORANGERED", OrangeRed },
        { "ORCHID", Orchid },
        { "PALEGOLDENROD", PaleGoldenrod },
        { "PALEGREEN", PaleGreen },
        { "PALETURQUOISE", PaleTurquoise },
        { "PALEVIOLETRED", PaleVioletRed },
        { "PAPAYAWHIP", PapayaWhip },
        { "PEACHPUFF", PeachPuff },
        { "PERU", Peru },
        { "PINK", Pink },
        { "PLUM", Plum },
        { "POWDERBLUE", PowderBlue },
        { "PURPLE", Purple },
        { "REBECCAPURPLE", RebeccaPurple },
        { "RED", Red },
        { "ROSYBROWN", RosyBrown },
        { "ROYALBLUE", RoyalBlue },
        { "SADDLEBROWN", SaddleBrown },
        { "SALMON", Salmon },
        { "SANDYBROWN", SandyBrown },
        { "SEAGREEN", SeaGreen },
        { "SEASHELL", Seashell },
        { "SIENNA", Sienna },
        { "SILVER", Silver },
        { "SKYBLUE", SkyBlue },
        { "SLATEBLUE", SlateBlue },
        { "SLATEGRAY", SlateGray },
        { "SNOW", Snow },
        { "SPRINGGREEN", SpringGreen },
        { "STEELBLUE", SteelBlue },
        { "TAN", Tan },
        { "TEAL", Teal },
        { "THISTLE", Thistle },
        { "TOMATO", Tomato },
        { "TRANSPARENT", Transparent },
        { "TURQUOISE", Turquoise },
        { "VIOLET", Violet },
        { "WEBGRAY", WebGray },
        { "WEBGREEN", WebGreen },
        { "WEBMAROON", WebMaroon },
        { "WEBPURPLE", WebPurple },
        { "WHEAT", Wheat },
        { "WHITE", White },
        { "WHITESMOKE", WhiteSmoke },
        { "YELLOW", Yellow },
        { "YELLOWGREEN", YellowGreen }
    };

#pragma warning disable CS1591 // Disable warning: "Missing XML comment for publicly visible type or member"
    public static KColor AliceBlue => new(0xF0F8FFFF);
    public static KColor AntiqueWhite => new(0xFAEBD7FF);
    public static KColor Aqua => new(0x00FFFFFF);
    public static KColor Aquamarine => new(0x7FFFD4FF);
    public static KColor Azure => new(0xF0FFFFFF);
    public static KColor Beige => new(0xF5F5DCFF);
    public static KColor Bisque => new(0xFFE4C4FF);
    public static KColor Black => new(0x000000FF);
    public static KColor BlanchedAlmond => new(0xFFEBCDFF);
    public static KColor Blue => new(0x0000FFFF);
    public static KColor BlueViolet => new(0x8A2BE2FF);
    public static KColor Brown => new(0xA52A2AFF);
    public static KColor Burlywood => new(0xDEB887FF);
    public static KColor CadetBlue => new(0x5F9EA0FF);
    public static KColor Chartreuse => new(0x7FFF00FF);
    public static KColor Chocolate => new(0xD2691EFF);
    public static KColor Coral => new(0xFF7F50FF);
    public static KColor CornflowerBlue => new(0x6495EDFF);
    public static KColor Cornsilk => new(0xFFF8DCFF);
    public static KColor Crimson => new(0xDC143CFF);
    public static KColor Cyan => new(0x00FFFFFF);
    public static KColor DarkBlue => new(0x00008BFF);
    public static KColor DarkCyan => new(0x008B8BFF);
    public static KColor DarkGoldenrod => new(0xB8860BFF);
    public static KColor DarkGray => new(0xA9A9A9FF);
    public static KColor DarkGreen => new(0x006400FF);
    public static KColor DarkKhaki => new(0xBDB76BFF);
    public static KColor DarkMagenta => new(0x8B008BFF);
    public static KColor DarkOliveGreen => new(0x556B2FFF);
    public static KColor DarkOrange => new(0xFF8C00FF);
    public static KColor DarkOrchid => new(0x9932CCFF);
    public static KColor DarkRed => new(0x8B0000FF);
    public static KColor DarkSalmon => new(0xE9967AFF);
    public static KColor DarkSeaGreen => new(0x8FBC8FFF);
    public static KColor DarkSlateBlue => new(0x483D8BFF);
    public static KColor DarkSlateGray => new(0x2F4F4FFF);
    public static KColor DarkTurquoise => new(0x00CED1FF);
    public static KColor DarkViolet => new(0x9400D3FF);
    public static KColor DeepPink => new(0xFF1493FF);
    public static KColor DeepSkyBlue => new(0x00BFFFFF);
    public static KColor DimGray => new(0x696969FF);
    public static KColor DodgerBlue => new(0x1E90FFFF);
    public static KColor Firebrick => new(0xB22222FF);
    public static KColor FloralWhite => new(0xFFFAF0FF);
    public static KColor ForestGreen => new(0x228B22FF);
    public static KColor Fuchsia => new(0xFF00FFFF);
    public static KColor Gainsboro => new(0xDCDCDCFF);
    public static KColor GhostWhite => new(0xF8F8FFFF);
    public static KColor Gold => new(0xFFD700FF);
    public static KColor Goldenrod => new(0xDAA520FF);
    public static KColor Gray => new(0xBEBEBEFF);
    public static KColor Green => new(0x00FF00FF);
    public static KColor GreenYellow => new(0xADFF2FFF);
    public static KColor Honeydew => new(0xF0FFF0FF);
    public static KColor HotPink => new(0xFF69B4FF);
    public static KColor IndianRed => new(0xCD5C5CFF);
    public static KColor Indigo => new(0x4B0082FF);
    public static KColor Ivory => new(0xFFFFF0FF);
    public static KColor Khaki => new(0xF0E68CFF);
    public static KColor Lavender => new(0xE6E6FAFF);
    public static KColor LavenderBlush => new(0xFFF0F5FF);
    public static KColor LawnGreen => new(0x7CFC00FF);
    public static KColor LemonChiffon => new(0xFFFACDFF);
    public static KColor LightBlue => new(0xADD8E6FF);
    public static KColor LightCoral => new(0xF08080FF);
    public static KColor LightCyan => new(0xE0FFFFFF);
    public static KColor LightGoldenrod => new(0xFAFAD2FF);
    public static KColor LightGray => new(0xD3D3D3FF);
    public static KColor LightGreen => new(0x90EE90FF);
    public static KColor LightPink => new(0xFFB6C1FF);
    public static KColor LightSalmon => new(0xFFA07AFF);
    public static KColor LightSeaGreen => new(0x20B2AAFF);
    public static KColor LightSkyBlue => new(0x87CEFAFF);
    public static KColor LightSlateGray => new(0x778899FF);
    public static KColor LightSteelBlue => new(0xB0C4DEFF);
    public static KColor LightYellow => new(0xFFFFE0FF);
    public static KColor Lime => new(0x00FF00FF);
    public static KColor LimeGreen => new(0x32CD32FF);
    public static KColor Linen => new(0xFAF0E6FF);
    public static KColor Magenta => new(0xFF00FFFF);
    public static KColor Maroon => new(0xB03060FF);
    public static KColor MediumAquamarine => new(0x66CDAAFF);
    public static KColor MediumBlue => new(0x0000CDFF);
    public static KColor MediumOrchid => new(0xBA55D3FF);
    public static KColor MediumPurple => new(0x9370DBFF);
    public static KColor MediumSeaGreen => new(0x3CB371FF);
    public static KColor MediumSlateBlue => new(0x7B68EEFF);
    public static KColor MediumSpringGreen => new(0x00FA9AFF);
    public static KColor MediumTurquoise => new(0x48D1CCFF);
    public static KColor MediumVioletRed => new(0xC71585FF);
    public static KColor MidnightBlue => new(0x191970FF);
    public static KColor MintCream => new(0xF5FFFAFF);
    public static KColor MistyRose => new(0xFFE4E1FF);
    public static KColor Moccasin => new(0xFFE4B5FF);
    public static KColor NavajoWhite => new(0xFFDEADFF);
    public static KColor NavyBlue => new(0x000080FF);
    public static KColor OldLace => new(0xFDF5E6FF);
    public static KColor Olive => new(0x808000FF);
    public static KColor OliveDrab => new(0x6B8E23FF);
    public static KColor Orange => new(0xFFA500FF);
    public static KColor OrangeRed => new(0xFF4500FF);
    public static KColor Orchid => new(0xDA70D6FF);
    public static KColor PaleGoldenrod => new(0xEEE8AAFF);
    public static KColor PaleGreen => new(0x98FB98FF);
    public static KColor PaleTurquoise => new(0xAFEEEEFF);
    public static KColor PaleVioletRed => new(0xDB7093FF);
    public static KColor PapayaWhip => new(0xFFEFD5FF);
    public static KColor PeachPuff => new(0xFFDAB9FF);
    public static KColor Peru => new(0xCD853FFF);
    public static KColor Pink => new(0xFFC0CBFF);
    public static KColor Plum => new(0xDDA0DDFF);
    public static KColor PowderBlue => new(0xB0E0E6FF);
    public static KColor Purple => new(0xA020F0FF);
    public static KColor RebeccaPurple => new(0x663399FF);
    public static KColor Red => new(0xFF0000FF);
    public static KColor RosyBrown => new(0xBC8F8FFF);
    public static KColor RoyalBlue => new(0x4169E1FF);
    public static KColor SaddleBrown => new(0x8B4513FF);
    public static KColor Salmon => new(0xFA8072FF);
    public static KColor SandyBrown => new(0xF4A460FF);
    public static KColor SeaGreen => new(0x2E8B57FF);
    public static KColor Seashell => new(0xFFF5EEFF);
    public static KColor Sienna => new(0xA0522DFF);
    public static KColor Silver => new(0xC0C0C0FF);
    public static KColor SkyBlue => new(0x87CEEBFF);
    public static KColor SlateBlue => new(0x6A5ACDFF);
    public static KColor SlateGray => new(0x708090FF);
    public static KColor Snow => new(0xFFFAFAFF);
    public static KColor SpringGreen => new(0x00FF7FFF);
    public static KColor SteelBlue => new(0x4682B4FF);
    public static KColor Tan => new(0xD2B48CFF);
    public static KColor Teal => new(0x008080FF);
    public static KColor Thistle => new(0xD8BFD8FF);
    public static KColor Tomato => new(0xFF6347FF);
    public static KColor Transparent => new(0xFFFFFF00);
    public static KColor Turquoise => new(0x40E0D0FF);
    public static KColor Violet => new(0xEE82EEFF);
    public static KColor WebGray => new(0x808080FF);
    public static KColor WebGreen => new(0x008000FF);
    public static KColor WebMaroon => new(0x800000FF);
    public static KColor WebPurple => new(0x800080FF);
    public static KColor Wheat => new(0xF5DEB3FF);
    public static KColor White => new(0xFFFFFFFF);
    public static KColor WhiteSmoke => new(0xF5F5F5FF);
    public static KColor Yellow => new(0xFFFF00FF);
    public static KColor YellowGreen => new(0x9ACD32FF);
#pragma warning restore CS1591
}