﻿using KludgeBox.VFS;
using KludgeBox.VFS.FileSystems;

namespace KludgeBox.Tests.VFS.FileSystems;

// This class is a minimal implementation of the FileSystem interface based on MemoryFileSystem.
public class MinimalFileSystem : FileSystem
{
    private readonly MemoryFileSystem _fileSystem = new(false);
    public override bool IsReadOnly => false;

    public MinimalFileSystem()
    {
        Name = $"MinFS >> {_fileSystem.Name}";
    }
    
    public override FsDirectory CreateDirectory(string path)
    {
        var dir = _fileSystem.CreateDirectory(path);
        return new FsDirectory(this, dir.Path);
    }

    public override void DeleteDirectory(string path, bool recursive = false)
    {
        _fileSystem.DeleteDirectory(path, recursive);
    }

    public override FsFile CreateFile(string path)
    {
        var file = _fileSystem.CreateFile(path);
        return new FsFile(this, file.Path);
    }

    public override void DeleteFile(string path)
    {
        _fileSystem.DeleteFile(path);
    }

    public override Stream OpenRead(string path)
    {
        return _fileSystem.OpenRead(path);
    }

    public override Stream OpenWrite(string path)
    {
        return _fileSystem.OpenWrite(path);
    }

    public override FsFile[] GetFiles(string path)
    {
        return _fileSystem.GetFiles(path)
            .Select(dir => new FsFile(this, dir.Path))
            .ToArray();;
    }

    public override FsDirectory[] GetDirectories(string path)
    {
        return _fileSystem.GetDirectories(path)
            .Select(dir => new FsDirectory(this, dir.Path))
            .ToArray();
    }

    public override bool IsDirectory(string path)
    {
        return _fileSystem.IsDirectory(path);
    }

    public override bool IsFile(string path)
    {
        return _fileSystem.IsFile(path);
    }
}